#! /usr/bin/python3
import sys
import os

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

# pipi
from colorama import Fore
import typer

# local
from getlooks.utils import message, show_table
from getlooks.looks import looks_install, looks_list, looks_rm, looks_log, looks_update
from getlooks.core import DeskEnv


def see_help(arg: str = "", stop_here: bool = True):
    return message.error(
        "This command required arguments, use "
        + Fore.LIGHTYELLOW_EX
        + f"{arg} --help"
        + Fore.RED
        + " to see them"
        + Fore.RESET,
        stop_here,
    )


app = typer.Typer(
    help=f"{Fore.LIGHTGREEN_EX} Theme Installer for Gnome, Xfce4, Kde {Fore.RESET}"
)


@app.command()
def get(url: str = typer.Argument(None, help="looks or pling [URL] of theme/icon")):
    """
    | Install new UI themes/icons
    """

    if url is None or url == "":
        see_help("get")

    product = looks_install(url)
    looks_log(product)


@app.command()
def update(
    me: bool = typer.Option(False, "--me", help="Update tool itself"),
    themes: bool = typer.Option(
        False, "-t", "--themes", help="Update all gtk/icon themes"
    ),
):
    """
    | Update installed themes and icons via this tool
    """

    if me:
        os.system("pip install -U gnomelooks")
    elif themes:
        looks_update()
    else:
        see_help("update")


@app.command()
def rm(
    themes: bool = typer.Option(False, "-t", "--themes", help="To remove Themes"),
    icons: bool = typer.Option(False, "-i", "--icons", help="To remove Icons"),
):
    """
    | Remove installed themes and icons
    """
    if themes:
        looks_rm(themes=True)
    elif icons:
        looks_rm(icons=True)
    else:
        see_help("rm")


@app.command()
def askenv():
    """
    | ask deskenv
    """
    denv = DeskEnv()
    denv.env_prompt()


@app.command()
def ls(
    themes: bool = typer.Option(False, "-t", "--themes", help="To list Themes]"),
    icons: bool = typer.Option(False, "-i", "--icons", help="To list Icons]"),
):
    """
    | List installed themes and icons
    """
    print()
    if (themes or icons) != True:
        see_help("ls")

    _look_list = looks_list(themes=themes, icons=icons)[0]

    title = "Theme names"
    if icons:
        title = "Icon names"

    show_table(
        data=[{"id": i, "names": _look_list[i]} for i in range(len(_look_list))],
        title=title,
    )
    print()


if __name__ == "__main__":
    app()



#! /usr/bin/python3

import os
import json
import random
import shutil
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import AnyStr, List, Dict
import logging

# pipi
import requests
from colorama import Fore
from rich.console import Console
from rich.text import Text
from rich.table import Table
from rich.logging import RichHandler

# locals

# -- code --
this_file_path = Path(os.path.abspath(__file__)).resolve()


def _logger(debug_flag: str = ""):

    # message
    logger = logging.getLogger(__name__)

    if os.environ.get(debug_flag) != None:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.WARNING)

    handler = RichHandler(log_time_format="")
    logger.addHandler(handler)
    return logger


logger = _logger(debug_flag="TEST_THEME_INSTALLER")


def git_pull():
    """pull git repo"""

    if os.path.exists(str(this_file_path.parent) + "/.git"):
        os.chdir(this_file_path.parent)
        os.system("git pull origin main")


class JsonState:

    cache: Dict[str, Dict] = dict()
    cache_path: str = ""

    def add(self, key: str, data: dict, dublicate: bool = False):

        if not dublicate:
            if key not in self.cache.keys():
                self.cache[key] = data
        else:
            self.cache[key] = data

    def put(self, key: str, data: dict) -> bool:
        # , where:dict={}

        if len(self.cache) == 0:
            self.add(key, data)
            return True

        if key != None:
            keys = self.cache[key].keys()

        if not keys == data.keys():
            print("old keys: ", keys)
            print("new keys: ", data.keys())
            raise ValueError("previos key datakeys not match")

        self.cache[key] = data
        return True

    def load(self, cache_path: str):
        if not cache_path.split(".")[-1] == "json":
            raise ValueError("expected json file")

        elif not os.path.exists(cache_path):
            self.cache_path = cache_path
            return

        if not os.path.getsize(cache_path) == 0:

            with open(cache_path, "r") as c:
                data = json.loads(c.read())
                self.cache = data

        self.cache_path = cache_path

    def reload(self):
        self.load(self.cache_path)

    def save(self):
        parent_dir = os.path.dirname(self.cache_path)

        if not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

        with open(self.cache_path, "w") as c:
            data = json.dumps(self.cache, indent=4)
            c.write(data)
            print(f"Updated state: {self.cache_path}")
            return True


class Message:
    """Print message of type and color"""

    colors = [
        Fore.WHITE,
        # blue
        Fore.BLUE,
        Fore.LIGHTBLUE_EX,
        # green
        Fore.GREEN,
        Fore.LIGHTGREEN_EX,
        # cyan
        Fore.CYAN,
        Fore.LIGHTCYAN_EX,
        # red
        # Fore.RED,
        Fore.LIGHTRED_EX,
        # magenta
        Fore.MAGENTA,
        Fore.LIGHTMAGENTA_EX,
        # yellow
        Fore.YELLOW,
        Fore.LIGHTYELLOW_EX,
    ]

    def __init__(self):
        pass

    def error(self, message: str, stop_here=False):
        print(Fore.RED + "ERROR: " + message + Fore.RESET)
        if stop_here == True:
            exit(1)

    def sucesses(self, message: str):
        print(Fore.LIGHTGREEN_EX + message + Fore.RESET)

    def green(self, message: str):
        print(Fore.GREEN + message + Fore.RESET)

    def warning(self, message: str):
        print(Fore.YELLOW + "WARNING: " + message + Fore.RESET)

    def info(self, message: str):
        print(Fore.LIGHTMAGENTA_EX + "INFO: " + message + Fore.RESET)

    def title(self, message: str):
        print(Fore.LIGHTCYAN_EX + message + Fore.RESET)

    def random_color(self, message: str, **kwargs):
        r_color = random.choice(self.colors)
        print(r_color + message + Fore.RESET, **kwargs)
        return r_color


message = Message()


def ls(
    path: str,
    only_dir: bool = False,
    include_path: bool = False,
    ignore_items: list = [],
) -> list:
    """List files and directories of given path"""

    if not os.path.exists(path):
        raise Exception(f"Path: {path} not found")
    if only_dir:
        if include_path:
            p_ls = [
                path + f"/{file}"
                for file in os.listdir(path)
                if os.path.isdir(path + f"/{file}")
            ]
        else:
            p_ls = [
                file for file in os.listdir(path) if os.path.isdir(path + f"/{file}")
            ]
    else:
        if include_path:
            p_ls = [path + f"/{file}" for file in os.listdir(path)]
        else:
            p_ls = os.listdir(path)

    for item in ignore_items:
        if item in p_ls:
            p_ls.remove(item)

    p_ls.sort()
    return p_ls


def extract(path: str, at: str):
    """Extract tar file"""
    try:
        shutil.unpack_archive(path, at)
        message.sucesses(f"""Extracted {path}, at {at}""")
        return True
    except:
        logger.error(f"can't extract: {path}")
        raise Exception("Invalid file to extract")


def download(url: str, at: str):
    """"""

    file = requests.get(url, stream=True)

    if not os.path.exists(at):
        os.makedirs(at)

    file_name: str = url.split("/")[-1]
    if file.status_code == 200:
        with open(f"{at}/{file_name}", "wb") as fw:
            fw.write(file.content)
        message.info(f"""Downloaded: {file_name} at {at}""")
        return f"{at}/{file_name}"
    else:
        message.error("status_code {file.status_code}", stop_here=True)
        exit()


def dn_n_extract(url: str, at: str) -> list:
    """Download `url` and extract `at` path."""

    temp = TemporaryDirectory(prefix="looks_ex_", suffix="")
    temp_dn = TemporaryDirectory(prefix="looks_dn_", suffix="")

    # Download
    message.info(f"Downloading... \n")
    dp = download(url, at=temp_dn.name)

    # extract at tmp, and list folders
    extract(dp, temp.name)
    extracted_items: list = ls(temp.name, only_dir=True)

    for f in extracted_items:
        at_f = os.path.join(at, f)
        if os.path.exists(at_f):
            shutil.rmtree(at_f)
        shutil.move(os.path.join(temp.name, f), at_f)
        print(f"Moved to: {at_f}")

    temp_dn.cleanup()
    temp.cleanup()
    return extracted_items


def dict_list_tbl(items=list[dict], ignore_keys: list = []):
    keys = []
    data = []

    for item in items:
        _tmp: tuple = ()
        for key in [i for i in item.keys() if i not in ignore_keys]:

            if key not in keys:
                keys.append(key)
            _tmp += (str(item[key]),)
        data.append(_tmp)

    return keys, data


def show_table(data: List[Dict], ignore_keys: List = [], title: str = ""):
    """rich table"""

    text = Text(title, style="#D0FF5E bold")

    print()

    table = Table(title=text, style="#8782E9 bold")
    columns, rows = dict_list_tbl(data, ignore_keys)

    colors = {0: "#F0A45D bold", 1: "#E8678A", 2: "#8CC265", 3: "#76F6FF", 4: "#4AA5F0"}

    for count, col in enumerate(columns):
        color = colors[count % len(colors)]
        table.add_column(col, justify="left", style=color, no_wrap=True)
    for row in rows:
        table.add_row(*row)

    console = Console()
    console.print(table)



#! /usr/bin/python3

import re
import os
from pathlib import Path
import json
import base64
from datetime import datetime
from urllib.parse import unquote
from dataclasses import dataclass, field
from typing import Dict, List, Optional

# pipi
import requests
from bs4 import BeautifulSoup
from requests.api import patch

# local
from getlooks.utils import show_table, message
from getlooks.looks_path import DESK_THEME_PATH, STATE_PATH


path_for = {
    "GTK3/4 Themes": "themes",
    "Full Icon Themes": "icons",
    "Cursors": "cursors",
}


@dataclass
class FileInfo:
    """
    Content related file info details
    """

    id: str
    name: str
    type: str
    is_active: bool
    url: str
    size: str
    downloads: int
    changed_at: str
    extracted_files: Optional[List[str]] = field(default_factory=list[str])

    def __post_init__(self):
        if isinstance(self.url, str):
            self.url = unquote(self.url)

    def date(self) -> datetime:
        return datetime.strptime(self.changed_at, "%Y-%m-%d %H:%M:%S")


@dataclass
class ProductInfo:
    """
    Content related theme page details
    """

    type: str
    project_id: str
    author: str
    description: str
    title: str
    likes: str
    changed_at: str
    system_tags: List[str]
    files: List[FileInfo]
    version: Optional[str] = ""
    url: Optional[str] = field(default_factory=str)
    install_path: Optional[str] = field(default_factory=str)

    def date(self) -> datetime:
        return datetime.strptime(f"{self.changed_at}", "%Y-%m-%d %H:%M:%S")

    def __is_type(self, expect: str):
        type = path_for.get(self.type)
        if type != None and type == expect:
            return True
        else:
            return False

    def is_icon(self):
        return self.__is_type("icons")

    def is_theme(self):
        return self.__is_type("themes")

    def is_cursor(self):
        return self.__is_type("cursors")

    # def __post_init__(self):
    #     files = []
    #     for file in self.files:
    #         if not isinstance(FileInfo, file):
    #             _f = FileInfo(**file)
    #             files.append(_f)
    #         else:
    #             files.append(file)
    #     self.files = files


class DeskEnv:
    """
    Check desktop env:
    - gnome
    - kde
    - xfce4
    and set theme path based on above env
    """

    USER = os.environ.get("USER")
    SUDO_USER = os.environ.get("SUDO_USER")
    HOME = (SUDO_USER and f"/home/{SUDO_USER}") or os.environ.get("HOME")
    DESK_ENV = ""
    state_file = Path(f"{HOME}/{STATE_PATH}-{USER}/deskenv")
    valid_env = {
        "1": "gnome",
        "2": "kde",
        "3": "xfce",
    }

    def __init__(self):
        if os.path.isfile(self.state_file):
            with open(self.state_file, "r") as s:
                state = s.read()
                self.DESK_ENV = state.strip()

    def env_prompt(self):

        if not os.path.isdir(self.state_file.parent):
            os.makedirs(self.state_file.parent)

        message.error("Can't figureout desktop environment")
        message.title(
            "Which one is your desktop environment ?\n"
            "\nChoose you desktop environment id, \nExample: 1 for gnome\n"
        )
        show_table([self.valid_env], title="Select desktop environment")

        print("\nEnter id for your desktop environment to continue: ", end="")
        ans = int(input())
        if ans <= len(self.valid_env):
            self.DESK_ENV = self.valid_env[str(ans)]
            with open(self.state_file, "w") as denv:
                denv.write(f"{self.DESK_ENV}")
        else:
            exit(1)

    def check_env(self):
        """Updates self.DESK_ENV according to your desk env"""

        if len(self.DESK_ENV) != 0:
            return

        # read /proc/sched_debug
        XDG_DESK = os.environ.get("XDG_CURRENT_DESKTOP") or os.environ.get(
            "ORIGINAL_XDG_CURRENT_DESKTOP"
        )
        if XDG_DESK:
            # print(XDG_DESK)
            ps = f"{XDG_DESK}".lower()

            if re.search("gnome", ps):
                self.DESK_ENV = self.valid_env["1"]
            elif re.search("plasma|kde", ps):
                self.DESK_ENV = self.valid_env["2"]
            elif re.search("xfce", ps):
                self.DESK_ENV = self.valid_env["3"]
            else:
                self.env_prompt()
        else:
            self.env_prompt()

    def theme_path(self, theme_type):
        """
        set the path for themes
        `theme_type` -> `themes` or `icons` or `cursor`
        """

        USER = self.USER
        _d = DESK_THEME_PATH[self.DESK_ENV]

        # arg = (themes or icons or cursor)
        if theme_type in _d.keys():
            if USER == "root":
                path = _d[theme_type]["global"]
            else:
                path = _d[theme_type]["local"]

        elif theme_type == "kde_global" and self.DESK_ENV == "kde":
            if USER == "root":
                path = _d["global_theme"]["global"]
            else:
                path = _d["global_theme"]["local"]
        else:
            raise ValueError(f"got unexpected theme_type -> {theme_type} ")

        return path.replace("~", self.HOME)


def scrapGnomeLooks(url):
    """"""

    try:
        gnome_looks_page = requests.get(url, timeout=30)
    except requests.exceptions.ConnectTimeout:
        raise Exception(f"Connection timeout {url}")

    if gnome_looks_page.status_code == 404:
        message.error("Page responded 404 (File not found)", stop_here=True)
        exit()

    soup = BeautifulSoup(gnome_looks_page.text, "lxml")

    # where a variable contain json data of theme-files

    # product details
    pattern = r"productViewDataEncoded = '(ey.+)';var categoryId"

    # if api-json is not present at jspath, bruteforce for jspath
    script_child = [n for n in range(10, 26)]
    for blind_id in script_child:
        try:
            productData = soup.select_one(
                f"#od-body > script:nth-child({blind_id})"
            ).string
            p = re.search(pattern, productData)[1]
            p = base64.decode(p)

            if re.search(pattern, productData):
                EXIT_FLAG = False
                break
        except:
            EXIT_FLAG = True
            continue

        if EXIT_FLAG == True:
            message.error("variable productViewDataEncoded: not found in source code")
            exit(1)

    _p = base64.b64decode(p).decode("utf-8")

    # todict
    p_info = json.loads(_p)
    f_info = json.loads(requests.get(url + "/loadFiles").text)

    # dataclasses
    f_list: List[FileInfo] = []
    count = 0

    system_tags = [t["tag_name"] for t in p_info["systemTags"]]

    for _f in f_info["files"]:
        __t = {
            "id": f"{count}",
            # "g_id": _f["id"],
            "is_active": bool(int(_f["active"])),
            "name": _f["name"],
            "type": _f["type"],
            "url": _f["url"],
            "changed_at": _f["updated_timestamp"],
            "downloads": _f["downloaded_count_uk"],
            "size": f"{int(_f['size'])/1000000}"[:4] + " Mb",
        }
        if __t["is_active"]:
            f_list.append(FileInfo(**__t))
            count += 1

    __p_info = {
        "type": p_info["product"]["cat_title"],
        "project_id": p_info["product"]["project_id"],
        "author": p_info["product"]["username"],
        "description": p_info["product"]["description"],
        "title": p_info["product"]["title"],
        "likes": p_info["tabCnt"]["cntLikes"],
        "version": p_info["product"]["version"],
        "changed_at": p_info["product"]["changed_at"],
        "system_tags": system_tags,
        "files": f_list,
        "url": url,
    }

    # theme file

    # collect active files and ignore archived ones.
    return ProductInfo(**__p_info)


def product_state_cache(data: Dict[str, dict]) -> Dict[str, ProductInfo]:

    products: Dict[str, ProductInfo] = dict()
    for p in data:
        product = ProductInfo(**data[p])
        product.files = [FileInfo(**f) for f in data[p]["files"]]
        products[product.project_id] = product

    return products



STATE_PATH = ".config/looks-install"

DESK_THEME_PATH = {
    "gnome": {
        "themes": {
            "local": "~/.local/share/themes",
            "global": "/usr/share/themes",
        },
        "icons": {
            "local": "~/.local/share/icons",
            "global": "/usr/share/icons",
        },
        "cursors": {
            "local": "~/.local/share/icons",
            "global": "/usr/share/icons",
        },
    },
    "kde": {
        "themes": {
            "local": "~/.local/share/plasma/desktoptheme",
            "global": "/usr/share/plasma/desktoptheme",
        },
        "global_theme": {
            "local": "~/.local/share/plasma/look-and-feel",
            "global": "/usr/share/plasma/look-and-feel",
        },
        "icons": {
            "local": "~/.local/share/icons",
            "global": "/usr/share/icons",
        },
        "cursors": {
            "local": "~/.icons",
            "global": "/usr/share/icons",
        },
    },
    "xfce": {
        "themes": {
            "local": "~/.themes",
            "global": "/usr/share/themes",
        },
        "icons": {
            "local": "~/.icons",
            "global": "/usr/share/icons",
        },
        "cursors": {
            "local": "~/.icons",
            "global": "/usr/share/icons",
        },
    },
}



#! /usr/bin/python3

import dataclasses
import os
import shutil
from typing import List
from copy import deepcopy
from colorama import Fore

# local
from getlooks.looks_path import STATE_PATH
from getlooks.utils import JsonState, JsonState, dn_n_extract, show_table, logger
from getlooks.core import (
    ProductInfo,
    DeskEnv,
    scrapGnomeLooks,
    message,
    product_state_cache,
    path_for,
)

# -- code --

desk = DeskEnv()

if os.environ.get("TEST_THEME_INSTALLER") == "true":
    state_path = f"./test/state.json"
else:
    state_path = f"{desk.HOME}/{STATE_PATH}-{desk.USER}/state.json"


def get_path(product: ProductInfo) -> str:

    # print(product.type)

    if os.environ.get("TEST_THEME_INSTALLER") == "true":
        path = "./themes/" + product.type
    elif product.type == "Global Themes" and desk.DESK_ENV == "kde":
        path = desk.theme_path("kde_global")
    else:
        if product.type in path_for:
            path = desk.theme_path(path_for[product.type])
        else:
            message.warning("Can't identify product category, using current path.")
            path = "."
    return path


def state_update_rm(removed_items: List[str]):
    state = JsonState()

    state.load(state_path)
    _products = product_state_cache(state.cache)
    products = deepcopy(_products)

    for path in removed_items:
        file_name = path.split("/")[-1]
        for _id in _products:
            product = products[_id]
            if len(product.files) == 1 and len(product.files[0].extracted_files) == 1:
                if file_name in product.files[0].extracted_files:
                    products.pop(_id)
                    continue

            for _f in product.files:

                if file_name in _f.extracted_files:
                    if len(_f.extracted_files) == 1:
                        product.files.remove(_f)
                    else:
                        _f.extracted_files.remove(file_name)
    # update state
    for _id in products:
        logger.debug(products[_id])
        state.cache[_id] = dataclasses.asdict(products[_id])

    for _id in list(state.cache.keys()):
        if _id not in products.keys():
            state.cache.pop(_id)

    state.save()
    return state


def looks_list(themes: bool = False, icons: bool = False):
    state = JsonState()
    state.load(state_path)

    logger.debug(f"state_path: {state_path}")

    products = product_state_cache(state.cache)

    dirs: List[str] = []
    for p in products:
        product = products[p]
        if themes and product.is_theme():
            dirs += [file for f in product.files for file in f.extracted_files]
        elif icons and (product.is_icon() or product.is_cursor()):
            dirs += [file for f in product.files for file in f.extracted_files]

    return (dirs, products)


def looks_rm(themes: bool = False, icons: bool = False) -> List[str]:
    """Remove theme folder and update the state"""
    _look_list, products = looks_list(themes, icons)

    if len(_look_list) == 0:
        return []
    theme_path = products[list(products.keys())[0]].install_path

    show_table(
        [{"id": i, "names": _look_list[i]} for i in range(len(_look_list))],
        title="Remove theme",
    )
    print("Enter id, of file to remove")
    data: list = input().split(",")

    removed_items = []
    for f in data:
        li = _look_list[int(f)]
        try:
            item = f"{theme_path}/{li}"
            logger.debug(f"removing {item}")
            shutil.rmtree(path=item)
        except FileNotFoundError:
            ...
        logger.debug(li)
        removed_items.append(li)

    # remove item and update state
    logger.debug(removed_items)
    state_update_rm(removed_items)

    return removed_items


def looks_log(product: ProductInfo):

    state = JsonState()
    state.load(state_path)

    cache_products = product_state_cache(state.cache)

    if len(cache_products) != 0:
        for i in cache_products:
            if cache_products[i].url == product.url:
                final = []
                tmp = []
                for f in product.files + cache_products[i].files:
                    if f.name not in tmp:
                        tmp.append(f.name)
                        final.append(f)

                product.files = final
                _d = dataclasses.asdict(product)
                state.put(key=product.project_id, data=_d)
            else:
                _d = dataclasses.asdict(product)
                state.add(key=product.project_id, data=_d)
    else:
        _d = dataclasses.asdict(product)
        state.add(key=product.project_id, data=_d)

    state.save()


def looks_install(url, p_ids: List[str] = []) -> ProductInfo:

    """
    valid list for theme download
    [   "www.opendesktop.org/",
        "www.pling.com/",
        "store.kde.org/",
        "www.gnome-look.org/",
        "www.xfce-look.org/",    ]
    """

    if "gnome-look" in url:
        desk.DESK_ENV = "gnome"
    elif "kde.org" in url:
        desk.DESK_ENV = "kde"
    elif "xfce-look" in url:
        desk.DESK_ENV = "xfce"
    else:
        desk.check_env()

    product = scrapGnomeLooks(url)

    message.random_color(f"\nTheme-Name:  {product.title}", end="")
    message.random_color(f"\nType: {product.type}", end="")
    message.random_color(f"\tLikes: {product.likes}", end="")
    message.random_color(f"\tCreator: {product.author}\n")

    data = []
    for i in product.files:
        data.append(vars(i))
    show_table(
        data=data,
        ignore_keys=["type", "is_active", "url", "extracted_files"],
        title="Install Themes/Icons",
    )

    path = get_path(product)
    product.install_path = path

    print(f"{Fore.LIGHTBLUE_EX}\nEnter ids from above table,\nOf the files you want to download: {Fore.RESET}", end="")
    if len(p_ids) == 0:
        p_ids = input().split(",")
    files = []
    for p_id in p_ids:
        _id = int(p_id)
        theme_files = dn_n_extract(product.files[_id].url, product.install_path)

        # extra var
        product.files[_id].extracted_files = theme_files
        files.append(product.files[_id])

    product.files = files

    return product


def looks_update():

    state = JsonState()
    desk.check_env()

    state.load(state_path)

    if len(state.cache) == 0:
        message.error(
            "No themes installed,"
            + f"you need to install a theme first. check state: {state_path}"
            "",
            stop_here=True,
        )

    cache_products = product_state_cache(state.cache)

    for p_id in cache_products:
        _c_product = cache_products[p_id]
        product = scrapGnomeLooks(_c_product.url)
        product.install_path = _c_product.install_path

        message.sucesses(f"Fetching: {_c_product.url} ")

        _files: list = []
        for c_fi in _c_product.files:

            # latest product files -> _l_fi
            for l_fi in product.files:
                if l_fi.name == c_fi.name:

                    if l_fi.date() > c_fi.date():
                        _id = int(l_fi.id)
                        ex_files = dn_n_extract(
                            product.files[_id].url, product.install_path
                        )
                        l_fi.extracted_files = ex_files
                        _files.append(l_fi)
                    else:
                        _files.append(c_fi)
                        message.info(f"No updates: {c_fi.name} ")
                        continue

        product.files = _files

        _d = dataclasses.asdict(product)
        state.put(key=p_id, data=_d)

    state.save()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['getlooks', 'getlooks.tmp.test']

package_data = \
{'': ['*'],
 'getlooks': ['test/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/cinnamon/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/common-assets/menu/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/common-assets/misc/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/common-assets/panel/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/common-assets/switch/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/dark-assets/checkbox/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/dark-assets/menu/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/dark-assets/misc/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-Blue/cinnamon/dark-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gnome-shell/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gnome-shell/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gtk-2.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gtk-2.0/apps/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gtk-2.0/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gtk-3.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/gtk-4.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/metacity-1/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-Blue/xfwm4/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/cinnamon/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/common-assets/menu/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/common-assets/misc/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/common-assets/panel/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/common-assets/switch/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/dark-assets/checkbox/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/dark-assets/menu/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/dark-assets/misc/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar-v40/cinnamon/dark-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gnome-shell/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gnome-shell/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gtk-2.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gtk-2.0/apps/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gtk-2.0/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gtk-3.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/gtk-4.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/metacity-1/*',
              'themes/GTK3/4 Themes/Sweet-Ambar-v40/xfwm4/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/common-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/common-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/common-assets/panel/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar/cinnamon/common-assets/switch/*',
              'themes/GTK3/4 '
              'Themes/Sweet-Ambar/cinnamon/dark-assets/checkbox/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/dark-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/dark-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/cinnamon/dark-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gnome-shell/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gnome-shell/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gtk-2.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gtk-2.0/apps/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gtk-2.0/assets/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gtk-3.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/gtk-4.0/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/metacity-1/*',
              'themes/GTK3/4 Themes/Sweet-Ambar/xfwm4/*',
              'themes/GTK3/4 Themes/Sweet-Dark/*',
              'themes/GTK3/4 Themes/Sweet-Dark/assets/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/common-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/common-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/common-assets/panel/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/common-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/dark-assets/checkbox/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/dark-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/dark-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet-Dark/cinnamon/dark-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gnome-shell/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gnome-shell/assets/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gtk-2.0/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gtk-2.0/apps/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gtk-2.0/assets/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gtk-3.0/*',
              'themes/GTK3/4 Themes/Sweet-Dark/gtk-4.0/*',
              'themes/GTK3/4 Themes/Sweet-Dark/metacity-1/*',
              'themes/GTK3/4 Themes/Sweet-Dark/xfwm4/*',
              'themes/GTK3/4 Themes/Sweet/*',
              'themes/GTK3/4 Themes/Sweet/assets/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/common-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/common-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/common-assets/panel/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/common-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/dark-assets/checkbox/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/dark-assets/menu/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/dark-assets/misc/*',
              'themes/GTK3/4 Themes/Sweet/cinnamon/dark-assets/switch/*',
              'themes/GTK3/4 Themes/Sweet/gnome-shell/*',
              'themes/GTK3/4 Themes/Sweet/gnome-shell/assets/*',
              'themes/GTK3/4 Themes/Sweet/gtk-2.0/*',
              'themes/GTK3/4 Themes/Sweet/gtk-2.0/apps/*',
              'themes/GTK3/4 Themes/Sweet/gtk-2.0/assets/*',
              'themes/GTK3/4 Themes/Sweet/gtk-3.0/*',
              'themes/GTK3/4 Themes/Sweet/gtk-4.0/*',
              'themes/GTK3/4 Themes/Sweet/metacity-1/*',
              'themes/GTK3/4 Themes/Sweet/xfwm4/*',
              'tmp/*']}

install_requires = \
['beautifulsoup4',
 'colorama',
 'lxml',
 'requests',
 'rich>=12.4.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['gnomelooks = getlooks.cli:app']}

setup_kwargs = {
    'name': 'gnomelooks',
    'version': '0.1.7',
    'description': '',
    'long_description': '# Gnome-looks themes cli downloader\n\n**A cli-tool to install and update gnome based Icons, GTK, Cursor themes easily**\n\n### Supported desktop environments\n\n- **Gnone**\n- **Xfce**\n- **KDE Plasma**\n\n![image 1](./.github/images/get.png)\n\n## gnomelooks\n\n\n\n- To Install themes for current user\n        \n        gnomelooks get [THEME-URL]\n\n- To Install themes globally\n\n        sudo gnomelooks get [THEME-URL]\n\n### Installation\n\n    pip3 install -U gnomelooks\n\n## gnomelooks help Page\n\n    ~$ gnomelooks --help\n        Usage: gnomelooks [OPTIONS] COMMAND [ARGS]...\n\n        Theme Installer for Gnome, Xfce4, Kde \n\n        Options:\n        --install-completion  Install completion for the current shell.\n        --show-completion     Show completion for the current shell, to copy it or\n                                customize the installation.\n        --help                Show this message and exit.\n\n        Commands:\n        askenv  | ask deskenv\n        get     | Install new UI themes/icons\n        ls      | List installed themes and icons\n        rm      | Remove installed themes and icons\n        update  | Update installed themes and icons via this tool\n\n## update all themes and icons\n\nRun: `gnomelooks update --themes`\n',
    'author': 'Rishang',
    'author_email': 'rishangbhavsarcs@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
